@@ -6,7 +6,7 @@ Mojolicious::Plugin::LinkEmbedder - Convert a URL to embedded content
 
 =head1 VERSION
 
-0.10
+0.11
 
 =head1 DESCRIPTION
 
@@ -34,7 +34,7 @@ Or if you want full control:
             url => $link->url->to_string,
           },
         },
-        any => { text => "$link" },
+        any => { text => $link->to_embed },
       );
     });
   };
@@ -76,14 +76,59 @@ Or if you want full control:
 =cut
 
 use Mojo::Base 'Mojolicious::Plugin';
+use Mojo::Cache;
+use Mojo::JSON;
 use Mojo::Loader;
 use Mojo::UserAgent;
 use Mojolicious::Plugin::LinkEmbedder::Link;
 use constant DEBUG => $ENV{MOJO_LINKEMBEDDER_DEBUG} || 0;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 my $LOADER = Mojo::Loader->new;
 
+=head1 ATTRIBUTES
+
+=head2 cache_cb
+
+Holds a callback which will cache the results form the Link objects. An example
+of such an callback could be:
+
+  my %CACHE;
+  $self->cache_cb(
+    sub {
+      my $cb = pop;
+      my ($self, $url, $link) = @_;
+
+      if ($link) { # set
+        $CACHE{$url} = Mojo::JSON::encode_json($link);
+        $self->$cb;
+      }
+      else { # get
+        $self->$cb(Mojo::JSON::decode_json($CACHE{$url} || '{}'));
+      }
+    }
+  );
+
+=cut
+
+has cache_cb => sub {
+  return sub {
+    my $cb = pop;
+    my ($self, $url, $link) = @_;
+
+    if ($link) {
+      warn "SET $url\n" if DEBUG;
+      $self->_cache->set($url => Mojo::JSON::encode_json($link));
+      $self->$cb;
+    }
+    else {
+      warn "GET $url --- ", $self->_cache->get($url) || 'false', "\n" if DEBUG;
+      $self->$cb(Mojo::JSON::decode_json($self->_cache->get($url) || '{}'));
+    }
+  };
+};
+
+has _cache => sub { Mojo::Cache->new(keys => 200); };
 has _ua => sub { Mojo::UserAgent->new(max_redirects => 3) };
 
 =head1 METHODS
@@ -97,56 +142,84 @@ See L</SYNOPSIS>.
 sub embed_link {
   my ($self, $c, $url, $cb) = @_;
 
-  $url = Mojo::URL->new($url) unless ref $url;
-
-  if (my $type = lc $url->host) {
-    $type =~ s/^(?:www|my)\.//;
-    $type =~ s/\.\w+$//;
-    return $c if $self->_new_link_object($type => $c, {url => $url}, $cb);
-  }
-  if ($url->path =~ m!\.(?:jpg|png|gif)$!i) {
+  if ($url =~ m!\.(?:jpg|png|gif)\b!i) {
     return $c if $self->_new_link_object(image => $c, {url => $url}, $cb);
   }
-  if ($url->path =~ m!\.(?:mpg|mpeg|mov|mp4|ogv)$!i) {
+  if ($url =~ m!\.(?:mpg|mpeg|mov|mp4|ogv)\b!i) {
     return $c if $self->_new_link_object(video => $c, {url => $url}, $cb);
   }
 
-  return $self->_fallback($c, $url, $cb);
-}
-
-sub _fallback {
-  my ($self, $c, $url, $cb) = @_;
-
-  $self->_ua->head(
-    $url,
+  Scalar::Util::weaken($self);
+  $self->cache_cb->(
+    $self, $url,
     sub {
-      my ($ua, $tx) = @_;
-      my $ct = $tx->res->headers->content_type || '';
-
-      return if $ct =~ m!^image/!     and $self->_new_link_object(image => $c, {url => $url, _tx => $tx},  $cb);
-      return if $ct =~ m!^video/!     and $self->_new_link_object(video => $c, {url => $url, _tx => $tx},  $cb);
-      return if $ct =~ m!^text/html!  and $self->_new_link_object(html  => $c, {url => $url, _tx => $tx,}, $cb);
-      return if $ct =~ m!^text/plain! and $self->_new_link_object(text  => $c, {url => $url, _tx => $tx},  $cb);
-
-      warn "[LINK] New from $ct: Mojolicious::Plugin::LinkEmbedder::Link\n" if DEBUG;
-      return $c->$cb(Mojolicious::Plugin::LinkEmbedder::Link->new(url => $url));
+      my ($self, $data) = @_;
+      return $self->_new_link_object(undef => $c, $data, $cb) if $data and $data->{media_id};
+      return $self->_ua->head($url, sub { $_[1]->{input_url} = $url; $self->_learn($c, $_[1], $cb) });
     }
   );
 
   return $c;
 }
 
+sub _learn {
+  my ($self, $c, $tx, $cb) = @_;
+  my $ct = $tx->res->headers->content_type || '';
+  my $url = $tx->req->url;
+
+  return if $ct =~ m!^image/!     and $self->_new_link_object(image => $c, {url => $url, _tx => $tx}, $cb);
+  return if $ct =~ m!^video/!     and $self->_new_link_object(video => $c, {url => $url, _tx => $tx}, $cb);
+  return if $ct =~ m!^text/plain! and $self->_new_link_object(text  => $c, {url => $url, _tx => $tx}, $cb);
+
+  if (my $type = lc $url->host) {
+    $type =~ s/^(?:www|my)\.//;
+    $type =~ s/\.\w+$//;
+    return if $self->_new_link_object($type => $c, {url => $url, _tx => $tx}, $cb);
+  }
+  if ($ct =~ m!^text/html!) {
+    return if $self->_new_link_object(html => $c, {url => $url, _tx => $tx}, $cb);
+  }
+
+  warn "[LINK] New from $ct: Mojolicious::Plugin::LinkEmbedder::Link\n" if DEBUG;
+  $c->$cb(Mojolicious::Plugin::LinkEmbedder::Link->new(url => $url));
+}
+
 sub _new_link_object {
   my ($self, $type, $c, $args, $cb) = @_;
-  my $class = $self->{classes}{$type} or return;
+  my $class = $self->{classes}{$type} || $args->{class} || return;
   my $e = $LOADER->load($class);
 
   warn "[LINK] New from $type: $class\n" if DEBUG;
+  local $args->{ua} = $self->_ua;
+
+  if ($args->{url} and !ref $args->{url}) {
+    $args->{url} = Mojo::URL->new($args->{url});
+  }
 
   if (!defined $e) {
     my $link = $class->new($args);
-    $link->ua($self->_ua);
-    $link->learn($cb, $c, $link);
+
+    if ($link->{media_id}) {
+      $c->$cb($link);
+      return $class;
+    }
+
+    Mojo::IOLoop->delay(
+      sub {
+        my ($delay) = @_;
+        $link->learn($c, $delay->begin);
+      },
+      sub {
+        my ($delay) = @_;
+        return $delay->pass unless $args->{_tx}{input_url};
+        return $self->cache_cb->($self, $args->{_tx}{input_url}, $link, $delay->begin);
+      },
+      sub {
+        my ($delay) = @_;
+        $c->$cb($link);
+      },
+    );
+
     return $class;
   }
 
@@ -180,6 +253,7 @@ sub register {
   $self->{classes} = {
     '2play'        => 'Mojolicious::Plugin::LinkEmbedder::Link::Game::_2play',
     'beta.dbtv'    => 'Mojolicious::Plugin::LinkEmbedder::Link::Video::Dbtv',
+    'dbtv'         => 'Mojolicious::Plugin::LinkEmbedder::Link::Video::Dbtv',
     'blip'         => 'Mojolicious::Plugin::LinkEmbedder::Link::Video::Blip',
     'collegehumor' => 'Mojolicious::Plugin::LinkEmbedder::Link::Video::Collegehumor',
     'gist.github'  => 'Mojolicious::Plugin::LinkEmbedder::Link::Text::GistGithub',
@@ -205,6 +279,9 @@ sub register {
   if (my $route = $config->{route}) {
     $self->_add_action($app, $route);
   }
+  if (my $cb = $config->{cache_cb}) {
+    $self->cache_cb($cb);
+  }
 }
 
 sub _add_action {
